import 'package:freezed_annotation/freezed_annotation.dart';

part 'model.freezed.dart';

part 'model.g.dart';

extension FBModelEx on FBModel {
  ///只对自定义对象有用
  T? tryCase<T>() {
    return modelValue is T ? modelValue : null;
  }

  String? stringValue() => modelValue is String ? modelValue : null;

  num? intValue() => modelValue is num ? modelValue : null;

  Map<String, dynamic>? mapValue() =>
      modelValue is Map<String, dynamic> ? modelValue : null;

  bool? boolValue() => modelValue is bool ? modelValue : null;

  List<dynamic>? listValue() => modelValue is List<dynamic> ? modelValue : null;

  dynamic dynamicValue() => modelValue;

  bool get isNull => modelValue == null;

  dynamic get modelValue {
    return when(
        string: (value) => value,
        num: (value) => value,
        bool: (value) => value,
        list: (value) => value,
        json: (value) => value,
        dynamic: (value) => value,
        nil: () => null);
  }
}

@freezed
class FBModel with _$FBModel {
  const FBModel._();

  factory FBModel.string(String value) = StringData;

  factory FBModel.num(num value) = NumData;

  factory FBModel.bool(bool value) = BoolData;

  factory FBModel.list(List<dynamic> value) = ListData;

  factory FBModel.json(Map<String, dynamic> value) = JsonData;

  factory FBModel.dynamic(dynamic value) = DynamicData;

  const factory FBModel.nil() = NullData;

  factory FBModel.fromJson(Map<String, dynamic> json) =>
      _$FBModelFromJson(json);

  factory FBModel.createFrom(dynamic data) {
    if (data == null) {
      return const FBModel.nil();
    }
    var model = FBModel.dynamic(data);
    if (data is String) {
      model = FBModel.string(data);
    } else if (data is Map<String, dynamic>) {
      model = FBModel.json(data);
    } else if (data is List<dynamic>) {
      model = FBModel.list(data);
    } else if (data is num) {
      model = FBModel.num(data);
    } else if (data is bool) {
      model = FBModel.bool(data);
    }
    return model;
  }

  static FBModel freezedFromJson(dynamic v) => FBModel.createFrom(v);
}
